# Targeted Maximum Likelihood Estimation
# for binary point treatment, non-parametric estimation
# paramter of interest  = E_W[E(Y|A=1,W) - E(Y|A=0,W)]
# taking into account treatment (g_A) and missingness (g_M) mechanisms
# models or estimates for Q, g_A, g_M can be user-supplied or estimated internally using DSA
# as implemented, arguments to DSA are the same for all estimation procedures
# these can be user-supplied or set to default values
# maxorderint = 2, maxsumofpow=2, maxsize = 15
# Dmove=TRUE, Smove=TRUE, formula = Y~A, A forced into model.
# 
# August 16, 2009
# Susan Gruber, sgruber@berkeley.edu
# 
# for information see 
# M.J. van der Laan and D. Rubin. Targeted maximum likelihood learning. 
# The International Journal of Biostatistics, 2(1), 2006.

#-------------summary.tmle------------------
# return values of psi, variance, epsilon, 
# and models for Q, g_A, g_M, if available
#---------------------------------------	 
summary.tmle <- function(object,...) {
  if(identical(class(object), "tmle")){
		epsilon <- Qmodel <- Qcoef <- g_Amodel <- g_Acoef <- g_Mmodel <- g_Mcoef <- NULL
		Qterms <- g_Aterms <- g_Mterms <- ""
		if(!is.null(object$Q)){
		if (!is.null(object$Q$coef)) {
			Qcoef <- object$Q$coef	
			if(class(Qcoef) == "matrix"){
				Qterms <- colnames(Qcoef)
			} else {
				Qterms <- names(Qcoef)
			}
			Qmodel <- paste("Y ~ 1")
			if(length(Qterms) > 1) {
				Qmodel <- paste("Y ~ 1 +", paste(Qterms[-1], collapse =" + "))
			} 
		}}
		if(!is.null(object$g_A)){
		if (!is.null(object$g_A$coef)) {
			g_Acoef <- object$g_A$coef
			if(class(g_Acoef) == "matrix"){
				g_Aterms <- colnames(g_Acoef)
			} else {
				g_Aterms <- names(g_Acoef)
			}			
			g_Amodel <- paste("Y ~ 1")
			if(length(g_Aterms) > 1) {
				g_Amodel <- paste("Y ~ 1 +", paste(g_Aterms[-1], collapse =" + "))
			} 
		}}
		if(!is.null(object$g_M)){
		if (!is.null(object$g_M$coef)) {
			g_Mcoef <- object$g_M$coef
			if(class(g_Mcoef) == "matrix"){
				g_Mterms <- colnames(g_Mcoef)
			} else {
				g_Mterms <- names(g_Mcoef)
			}			
			g_Mmodel <- paste("Y ~ 1")
			if(length(g_Mterms) > 1) {
				g_Mmodel <- paste("Y ~ 1 +", paste(g_Mterms[-1], collapse =" + "))
			} 
		}}
		summary.tmle <- list(psi=object$psi, var=object$var, CI=object$CI, pvalue=object$pvalue, epsilon=object$epsilon, 
						Qmodel=Qmodel, Qterms=Qterms, Qcoef=Qcoef, g_Amodel=g_Amodel, g_Aterms=g_Aterms, g_Acoef=g_Acoef,
						g_Mmodel=g_Mmodel, g_Mterms=g_Mterms, g_Mcoef=g_Mcoef)
		class(summary.tmle) <- "summary.tmle"
	} else {
		stop("object must have class 'tmle'")
		summary.tmle <- NULL
	}
	return(summary.tmle)
}

#-------------print.summary.tmle------------------
print.summary.tmle <- function(object,...) {
  if(identical(class(object), "summary.tmle")){
	cat("\n Parameter Estimate: ", signif(object$psi,5), "\n")
	cat("\n Estimated Variance: ", signif(object$var,5), "\n")
	cat("\n\t\t   p-value:  ", ifelse(object$pvalue <= 2*10^-16, "<2e-16",signif(object$pvalue,5)), "\n")
	cat("\n 95% Conf Interval: ",paste("(", signif(object$CI[1],5), ", ", signif(object$CI[2],5), ")", sep=""),"\n")
  	cat("\n\n Initial parametric estimation of Q\n\n\t Model:\n\t\t", object$Qmodel)
  	cat("\n\n\t Coefficients: \n")
  	terms <- sprintf("%15s", object$Qterms)
  	extra <- ifelse(object$Qcoef >= 0, "  ", " ")
  	for(i in 1:length(object$Qcoef)){
  		cat("\t", terms[i], extra[i], object$Qcoef[i], "\n")
  	}
  	if(!(is.na(object$g_Acoef[1]))){	
  		cat("\n\n\n Estimation of g_A (treatment mechanism)\n\n\t Model:\n\t\t", object$g_Amodel, "\n")
  		cat("\n\n\t Coefficients: \n")
  		terms <- sprintf("%15s", object$g_Aterms)
  		extra <- ifelse(object$g_Acoef >= 0, "  ", " ")
  		for(i in 1:length(object$g_Acoef)){
  			cat("\t", terms[i], extra[i], object$g_Acoef[i], "\n")
  	}} 
  	if(!(is.na(object$g_Mcoef[1]))){		
  		cat("\n\n Estimation of g_M (missingness mechanism)\n\n\t Model:\n\t\t",object$g_Mmodel, "\n")
  		cat("\n\n\t Coefficients: \n")
  		terms <- sprintf("%15s", object$g_Mterms)
  		extra <- ifelse(object$g_Mcoef >= 0, "  ", " ")
  		for(i in 1:length(object$g_Mcoef)){
  			cat("\t", terms[i], extra[i], object$g_Mcoef[i], "\n")
  	}} 
  } else {
  	 stop("Error calling print.summary.tmle. Object needs to have class 'summary.tmle'")
  }
}

#-------------print.tmle------------------
print.tmle <- function(object,...) {
	if(identical(class(object), "tmle")){
		cat("\n Parameter Estimate: ", signif(object$psi,5), "\n")
		cat("\n Estimated Variance: ", signif(object$var,5), "\n")
		cat("\n\t\t   p-value:  ", ifelse(object$pvalue <= 2*10^-16, "<2e-16",signif(object$pvalue,5)), "\n")
		cat("\n 95% Conf Interval: ",paste("(", signif(object$CI[1],5), ", ", signif(object$CI[2],5), ")", sep=""),"\n") 
	} else {
 		stop("Error calling print.tmle. Object needs to have class 'tmle'")
 }}

#-------------.verify_args------------------	 
.verify_args <- function(Y,A,W,Delta){
	ok1 <- length(Y) == length(A) & length(A) == nrow(W)
	ok2 <- all(A[!is.na(A)] %in% 0:1)
	if (!ok1) {warning("'Y', 'A', 'W' must contain the same number of observations")}
	if (!ok2) {warning("'A' must be binary (0,1)")}
	return(ok1&ok2)
}
  
#-----------.set_DSAargs----------------
.set_DSAargs <- function(DSAargs, wts){
  	if(is.null(DSAargs$maxsumofpow)){DSAargs$maxsumofpow <- 2}
  	if(is.null(DSAargs$maxorderint)){DSAargs$maxorderint <- 2}
  	if(is.null(DSAargs$maxsize)) {DSAargs$maxsize <- 15} # arbitrary limit
  	if(is.null(DSAargs$Dmove)) {DSAargs$Dmove <- FALSE}
  	if(is.null(DSAargs$Smove)) {DSAargs$Smove <- FALSE}
  	if(is.null(DSAargs$vfold)) {DSAargs$vfold <- 5}
  	if(is.null(DSAargs$formula)){DSAargs$formula <- Y~A}
  	if(is.null(DSAargs$family)){DSAargs$family <- "gaussian"}
  	if(is.null(DSAargs$wts)) {DSAargs$wts <- matrix(data=rep(wts, DSAargs$vfold+1), byrow=TRUE, nrow=DSAargs$vfold+1)}  # find out if this is correct
  	if(is.null(DSAargs$nsplits)) {DSAargs$nsplits <- 1}
  	if(is.null(DSAargs$silent)) {DSAargs$silent <- -1}
  	return(DSAargs)
}
  
#-----------function .logit---------
# convert probability to .logit
# truncate probability passed in
#-------------------------------
.logit <- function(x){
        x[x>1] <-1
        x[x<0] <-0
        return(-log(1/x - 1))
}

#-----------estimate_Q----------------
# figure out if Q is one of three things:
# 1.  a matrix of values, QAW, Q1W, Q0W
# 2.  a model to use glm on 
# 3.  null - estimate with DSA if available, otherwise main terms with glm
# returns matrix of linear predictors for Q(A,W), Q(1,W), Q(0,W)
#----------------------------------------
estimate_Q <- function (Q, DSAargs, Y,A,W, Delta, family, wts, id) {
	m <- NULL
  	if(is.matrix(Q)){ 
  	  if (family == "binomial") {Q <- .logit(Q)}
  	  coef <- NA
  	} else {
  	  if (is.null(Q)){
  		if(require(DSA)){
  			DSAargs <- .set_DSAargs(DSAargs, wts)
  			m <- DSA(formula=DSAargs$formula, data=data.frame(Y,A,W)[Delta==1,], 
  				    weights=DSAargs$wts[,Delta==1], id=id[Delta==1], 
  					maxsumofpow=DSAargs$maxsumofpow, maxorderint=DSAargs$maxorderint, 
  					maxsize=DSAargs$maxsize, Dmoves=DSAargs$Dmove, Smove=DSAargs$Smove, 
  					family=family, candidate.rank=DSAargs$candidate.rank, 
  					rank.cutoffs = DSAargs$rank.cutoffs, usersplits=DSAargs$usersplits, 
  					userseed=DSAargs$userseed, vfold=DSAargs$vfold, nsplits=DSAargs$nsplits, 
  					silent=DSAargs$silent )
  		} else {
  			warning("DSA not found, running main terms regression for 'Q' using glm")
  			form <- paste("Y~A", paste(colnames(W), collapse = "+"), sep="+")
  			m <- glm(form, family=family, data=data.frame(Y,A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1)
  		}
  	  } else {
  	  	form <- try(as.formula(Q))
  	  	if(class(form)== "formula") {
  	  		m <- glm(form,  family=family, data=data.frame(Y,A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1)
  	  	} else {
  	  		warning("invalid formula supplied, running main terms regression for 'Q' using glm")
  	  		form <- paste("Y~A", paste(colnames(W), collapse = "+"), sep="+")
  			m <- glm(form, family=family, data=data.frame(Y,A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1)
  		}
  	  }		  		
  	  QAW <- predict(m, newdata=data.frame(Y,A,W))
  	  Q1W <- predict(m, newdata=data.frame(Y,A=1,W))
  	  Q0W <- predict(m, newdata=data.frame(Y,A=0,W))
  	  Q <- cbind(QAW, Q1W, Q0W)	
  	  coef <- coef(m)
    }
    return(list(Q=Q, coef=coef, type=class(m)[1]))
}

#-----------estimate_g----------------
# Estimate any factor of g
#----------------------------------------
estimate_g <- function (g, DSAargs,A,W, Delta, wts, id) {
  m <- NULL
  if (!is.numeric(g)){
	if (all(A==A[1])) {
		g1W <- 1
		coef<- NA
	} else {
  	  if (is.null(g)){
  		if(require(DSA)){
  			DSAargs <- .set_DSAargs(DSAargs, wts)
  			m <- DSA(formula=DSAargs$formula, data=data.frame(A,W)[Delta==1,], 
  				    weights=DSAargs$wts[,Delta==1], id=id[Delta==1], 
  					maxsumofpow=DSAargs$maxsumofpow, maxorderint=DSAargs$maxorderint, 
  					maxsize=DSAargs$maxsize, Dmoves=DSAargs$Dmove, Smove=DSAargs$Smove, 
  					family="binomial", candidate.rank=DSAargs$candidate.rank, 
  					rank.cutoffs = DSAargs$rank.cutoffs, usersplits=DSAargs$usersplits, 
  					userseed=DSAargs$userseed, vfold=DSAargs$vfold, nsplits=DSAargs$nsplits, 
  					silent=DSAargs$silent )
  		} else {
  			warning("DSA not found, running main terms regression for 'g' using glm")
			form <- paste("A~1", paste(colnames(W), collapse = "+"), sep="+")  
			m <- glm(form, family="binomial", data=data.frame(A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1)
  		}
  	  } else {
  	  	form <- try(as.formula(g))
  	  	if(class(form)== "formula") {
  	  		form <- update.formula(form, A~.)
  	  		m <- try(glm(form,  family="binomial", data=data.frame(A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1))
  	  		if (class(m)[1]=="try-error"){
  	  		warning("invalid formula supplied, running main terms regression for 'g' using glm")
  	  		form <- paste("A~1", paste(colnames(W), collapse = "+"), sep="+")
  	  		m <- glm(form, family="binomial", data=data.frame(A,W, wts, Delta), weights=wts,na.action=na.exclude, subset=Delta==1)
  	  		}
  	  	} else {
  	  		form <- paste("A~1", paste(colnames(W), collapse = "+"), sep="+")
  	  		m <- glm(form, family="binomial", data=data.frame(A,W, wts, Delta), weights=wts, na.action=na.exclude, subset=Delta==1)
  	  	}
  	  }
  	  g1W <- predict(m, newdata=data.frame(A,W,wts), type="response")
  	  coef <- m$coef
  	}
 } else {
 	g1W <- g
 	coef <- NA
 }		  		
 return(list(g1W=g1W, coef=coef, type=class(m)[1]))
}

#-------------------------------tmle----------------------------------------
# estimate marginal treatment effect for binary point treatment
# accounting for missing outcomes.
# arguments:
# Y - outcome
# A - binary treatment indicator, 1-treatment, 0 - control
# W - vector, matrix or dataframe containing baseline covariates
# Delta - indicator of missing outcome or treatment assignment.  1 - observed, 0 - missing
# id - id identifying repeated measures
# Q - E(Y|A,W), specified in one of three ways:
#	1. NULL - defaults to DSA estimation of E(Y|A=a, W), with A forced into the model
#	2. matrix of values containing three columns.  1: E(Y|A=a,W), 2: E(Y|A=1,W), 3: E(Y|A=0,W)
#	3. formula for estimation of E(Y|A, W), suitable for call to glm
# g_A - binary treatment mechanism, specified in one of three ways:
#	1. NULL - defaults to DSA estimation of P(A=1|W)
#	2. vector of values P(A=1|W)
#	3. formula for estimation of P(A=1,W), suitable for call to glm
# g_M - missingness mechanism, specified in one of three ways:
#	1. NULL - defaults to DSA estimation of P(Delta=1|W)
#	2. vector of values P(Delta=1|W)
#	3. formula for estimation of P(Delta=1,W), suitable for call to glm
# wts - optional weights on observations
# DSAargs - optional settings for DSA estimation
# 	defaults:  maxsumofpow = 2, maxorderint = 2, maxsize=min(2*ncol(W),15) (model size capped at 15), 
#			 vfold = 5, nsplits=1, Dmove=FALSE, Smove=FALSE
# family - family specification for regression models, defaults to gaussian
#	Returns
#	psi, treatment effect estimate,
#   pvalue, 2-sided p-value 
#	var - estimated variance of parameter estimate,
#	epsilon - coefficient used in targeting step
#	coefficients and predicted values for Q_n^0(A,W), g_A(1,W), g_M(1,A,W)
#-------------------------------------------------------------------------------

tmle <- function(Y,A,W,Delta=rep(1,length(Y)), id=1:length(Y), Q=NULL, g_A=NULL, g_M=NULL, wts=rep(1, length(Y)), DSAargs=NULL, family="gaussian", epsilon=NULL,...) {
	psi.tmle <- varIC <- CI <- pvalue <- NA
	W <- as.matrix(W)
	if(.verify_args(Y,A,W,Delta)){
		Q <- estimate_Q(Q, DSAargs, Y,A,W, Delta, family, wts, id)
		DSAargs$formula <- A~1
 	 	g_A <- estimate_g(g_A, DSAargs, A, W, Delta, wts, id) 
 	 	g_M <- estimate_g(g_M, DSAargs, A=Delta, W, Delta=rep(1,nrow(W)), wts, id) 
  		g1W <- g_A$g1W 
  		h <- h1W <- 1/g1W * Delta/g_M$g1W
  		h0W <- -1/(1-g1W) *  Delta/g_M$g1W
  		h[A==0] <- h0W[A==0]
  		
  		if(is.null(epsilon)) {
  			epsilon <- coef(glm(Y~-1 + offset(Q$Q[,1]) + h, family=family, weights=wts, subset=Delta==1))
  		} 
  	
  		QAW <- Q$Q[,1] + epsilon*h
  		Q1W <- Q$Q[,2] + epsilon*h1W
  		Q0W <- Q$Q[,3] + epsilon*h0W
  	
  		if (identical(family, binomial) | identical(family,"binomial")) {
  			QAW <- plogis(QAW)
  			Q1W <- plogis(Q1W)
  			Q0W <- plogis(Q0W)
  		}
  		psi.tmle <- mean(Q1W) - mean(Q0W)
  		Y[is.na(Y)] <- QAW[is.na(Y)] # keeps arithmetic from failing
  		IC <- (Y-QAW)*h*Delta + Q1W - Q0W - psi.tmle
    	IC <- as.vector(by(IC, id, mean))
    	IC[is.nan(IC)|is.infinite(IC)] <- Inf
    	varIC <- var(IC)
    	var.psi = varIC/length(unique(id))
    	CI <- c(psi.tmle-1.96*sqrt(var.psi), psi.tmle+1.96*sqrt(var.psi))
    	pvalue <- 2*pnorm(-abs(psi.tmle/sqrt(var.psi)))
  	}
	Qcounter <- cbind(Q1W, Q0W)
	colnames(Qcounter) <- c("Q1W", "Q0W")
  	returnVal <- list(psi=psi.tmle, var = var.psi, pvalue=pvalue, CI=CI, epsilon=epsilon, Q=Q, g_A=g_A, g_M=g_M, Qcounterfactual=Qcounter)
  
  	class(returnVal) <- "tmle"
  	return(returnVal)
}


