\name{tmle}
\Rdversion{1.1}
\alias{tmle}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Targeted Maximum Likelihood Estimation of Additive Treatment Effect (Lite)}
\description{
Carries out a simplified version of targeted maximum likelihood estimation of marginal additive treatment effect of a binary point treatment on an outcome. This parameter is defined as E(E(Y|A=1,W) - E(Y|A=0,W)), where  \code{Y} is a continuous or binary outcome variable, \code{A} is a binary treatment variable, (A=1 for treatment, A=0 for control), and \code{W} is a matrix or dataframe of baseline covariates.    The \code{tmle} function is minimally called with arguments \code{(Y,A,W)}.  Missingness in the outcome is accounted for in the estimation procedure, if optional missingness indicator \code{Delta} is supplied.}
\usage{
tmle(Y, A, W, Delta = rep(1, length(Y)), id = 1:length(Y),
Q = NULL, g_A = NULL, g_M = NULL, wts = rep(1, length(Y)), 
DSAargs = NULL, family = "gaussian", epsilon = NULL)
}
\arguments{
  \item{Y}{continuous or binary outcome variable}
  \item{A}{binary treatment indicator, \code{1} - treatment, \code{0} - control}
  \item{W}{vector, matrix, or dataframe containing baseline covariates}
  \item{Delta}{indicator of missing outcome or treatment assignment.  \code{1} - observed, \code{0} - missing}
  \item{id}{id identifying repeated measures}
  \item{Q}{E(Y|A,W), the \code{Q} portion of the likelihood, specified in one of three ways: \cr
  \code{NULL} specifies \code{DSA} estimation of E(Y|A=a, W), with  \code{A} forced into the model (default). \cr
     matrix of values, one row per observation, three columns: E(Y|A=a,W), E(Y|A=1,W), E(Y|A=0,W). \cr
    formula for estimation of E(Y|A, W), suitable for call to \code{glm}}.
  \item{g_A}{binary treatment mechanism, specified in one of three ways:  \cr
     \code{NULL} defaults to \code{DSA} estimation of P(A=1|W)  \cr
      vector of values P(A=1|W), one entry per observation  \cr
     formula for estimation of P(A=1,W), suitable for call to \code{glm}}.
  \item{g_M}{missingness mechanism, specified in one of three ways:  \cr
     \code{NULL}  defaults to \code{DSA} estimation of P(Delta=1|W)  \cr
     vector of values P(Delta=1|W), one entry per observation \cr  
     formula for estimation of P(Delta=1,W), suitable for call to \code{glm}}.
  \item{wts}{optional weights on observations. Defaults to unweighted}
  \item{DSAargs}{optional settings for \code{DSA} estimation. See \code{DSA} help files for further information.  Default settings are:  
    \code{maxsumofpow = 2, maxorderint = 2, maxsize=15, vfold = 5,  
    nsplits=1,  
    Dmove=FALSE,  
    Smove=FALSE.}}
\item{family}{family specification for working regression models, generally \sQuote{gaussian} for continuous outcomes (default), \sQuote{binomial} for binary outcomes.}
 \item{epsilon}{advanced option. Ordinarily this argument should not be specified, but can optionally be set to 0 to circumvent the targeting step.}
}
\value{
  \item{psi}{additive treatment effect estimate} 
  \item{var}{variance of estimate, based on the influence curve}
  \item{pvalue}{two-sided p-value}
  \item{CI}{95\% confidence interval}
  \item{epsilon}{MLE for coefficient used in targeting step}
  \item{Q}{initial estimate of \code{Q} portion of the likelihood. \code{Q$coefficients} are the coefficients for the model for \code{Q} selected by \code{DSA} or specified by the user.  \code{Q$Q} is an \code{nx3} matrix, where \code{n} is the number of observations.  Columns contain targeted predicted values for \code{Q(A,W),Q(1,W),Q(0,W)}, respectively.  \code{Q$type} is method for estimating \code{Q}, \code{NULL}- user supplied, \sQuote{DSA}, or \sQuote{glm}}
  \item{g_A}{treatment mechanism estimate. A list with two items: \code{g_A$coefficients} the coefficients for the model for \code{g_A} selected by \code{DSA} or specified by the user.  \code{g_A$g1W} contains values of P(A=1|W) for each observation}
\item{g_M}{missingness mechanism estimate. A list with two items: \code{g_M$coefficients} the coefficients for the model for \code{g_M} selected by \code{DSA} or specified by the user.  \code{g_M$g1W} contains values of P(Delta=1|A,W) for each observation}
\item{Qcounterfactual}{targeted estimate of counterfactual outcomes \code{Q(1,W),Q(0,W)}}
}
\references{
1. Mark J. van der Laan and Daniel Rubin (2006), \dQuote{Targeted Maximum Likelihood Learning}. \emph{The International Journal of Biostatistics}, 2(1). \url{http://www.bepress.com/ijb/vol2/iss1/11/}

2. Susan Gruber and Mark J. van der Laan (2009), \dQuote{Targeted Maximum Likelihood Estimation: A Gentle Introduction}. \emph{U.C. Berkeley Division of Biostatistics Working Paper Series}.  Working Paper 252.  \url{http://www.bepress.com/ucbbiostat/paper252}

3.  Mark J. van der Laan, Sherri Rose, Susan Gruber editors, (2009) \dQuote{Readings in Targeted Maximum Likelihood Estimation} . \emph{U.C. Berkeley Division of Biostatistics Working Paper Series}.  Working Paper xxx. \url{http://www.bepress.com/ucbbiostat}

4. Sandra E. Sinisi and Mark J. van der Laan, (2004). \dQuote{Loss-Based
  Cross-Validated Deletion/Substitution/Addition Algorithms in Estimation}.
  \emph{U.C. Berkeley Division of Biostatistics Working Paper
  Series}. Working Paper 143.
  \url{http://www.bepress.com/ucbbiostat/paper143}
}

\author{Susan Gruber \email{sgruber@berkeley.edu}, in collaboration with Mark van der Laan.}


\seealso{
\code{\link{summary.tmle}},
\code{\link{estimate_Q}},
\code{\link{estimate_g}},
\code{\link[DSA]{DSA}}
}
\examples{
library(tmleLite)

# generate data
  n <- 500
  W <- matrix(rnorm(n*3), ncol=3)
  A <- rbinom(n,1, 1/(1+exp(-(.1*W[,1] - .1*W[,2] + .5*W[,3]))))
  Y <- A + 2*W[,1] + W[,3] + W[,2]^2 + rnorm(n)
  colnames(W) <- paste("W",1:3, sep="")

# Example 1. Simplest function invocation 
# DSA called to estimate Q, g_A.
# Because Delta is not supplied, all outcomes are known to be measured.
#   
  result1 <- tmle(Y,A,W)
  summary(result1)

# Example 2: Binary outcome 
# DSA called to estimate Q
# known treatment mechanism, g_A, is user supplied 
# 
  A.ex2 <- rbinom(n,1,.5)
  Y.ex2 <- A.ex2 + 2*W[,1] + W[,3] + W[,2]^2 + rnorm(n)
  result2 <- tmle(Y=Y.ex2,A=A.ex2,W, g_A =rep(.5, length(Y)))
  summary(result2)

# Example 3: 
# User-supplied (misspecified) model for Q, 
# DSA called to estimate g_A, g_M
# approx. 20% missing, MAR

  Delta <- rbinom(n, 1, 1/(1+exp(-(1.7-1*W[,1]))))
  result3 <- tmle(Y,A,W, Delta=Delta, Q=Y~A+W1+W2+W3)
  summary(result3)
}
